@@ -3,7 +3,7 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Dict, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, Optional, Tuple, Union
 
 if TYPE_CHECKING:
     from aiohttp import ClientResponse
@@ -15,6 +15,7 @@ __all__: Tuple[str, ...] = (
     'InternalServerError',
     'RateLimited',
     'UserNotFound',
+    'PlatformNotFound',
     'OAuth2Unauthorized',
 )
 
@@ -22,9 +23,9 @@ __all__: Tuple[str, ...] = (
 class HTTPException(Exception):
     """Base exception class for all HTTP related errors."""
 
-    def __init__(self, response: ClientResponse, message: Union[str, Dict[str, Any]]) -> None:
+    def __init__(self, response: ClientResponse, message: Optional[Union[str, Dict[str, Any]]]) -> None:
         self.response: ClientResponse = response
-        self.message: Union[str, Dict[str, Any]] = message
+        self.message: Optional[Union[str, Dict[str, Any]]] = message
         self.status: int = response.status
         super().__init__(message)
 
@@ -50,7 +51,7 @@ class InternalServerError(HTTPException):
 class RateLimited(HTTPException):
     """Exception that's thrown when the HTTP request returns a 429 status code."""
 
-    def __init__(self, response: ClientResponse, message: str) -> None:
+    def __init__(self, response: ClientResponse, message: Optional[Union[str, Dict[str, Any]]]) -> None:
         self.retry_after = message.get('retry_after', 0) if isinstance(message, dict) else 0
         super().__init__(response, message)
 
@@ -63,6 +64,14 @@ class UserNotFound(Exception):
         self.message = message
 
 
+class PlatformNotFound(Exception):
+    """Exception that's thrown when the platform is not found in the database."""
+
+    def __init__(self, message: str) -> None:
+        super().__init__(message)
+        self.message = message
+
+
 class OAuth2Unauthorized(Exception):
     """Exception that's thrown when the OAuth2 token is invalid."""
 
